-- TestBench Template 

  LIBRARY ieee;
  USE ieee.std_logic_1164.ALL;
  USE ieee.numeric_std.ALL;

  library work;
  use work.pack_xtras.all;
  
  ENTITY testbench IS
		generic (N: INTEGER:= 8;   -- Number of summands
					B: INTEGER:= 8);  -- Number of bits per summand: B
  END testbench;

  ARCHITECTURE behavior OF testbench IS 

		component adder_tree
			port ( clock, resetn: in std_logic;
					 X_in: in std_logic_2d(N-1 downto 0, B-1 downto 0);
					 Yo: out std_logic_vector (B+ceil_log2(N) -1 downto 0));
		end component;

		type chunk is array (N-1 downto 0) of std_logic_vector(B-1 downto 0);
	   signal X: chunk;
	
      signal clock, resetn: std_logic:= '0';
		signal X_in: std_logic_2d(N-1 downto 0, B-1 downto 0);
		signal Yo: std_logic_vector(B+ceil_log2(N) -1 downto 0);          

		constant PERIOD : time := 20 ns; -- 50 MHz clock
		constant DUTY_CYCLE : real := 0.5;
		constant OFFSET : time := 100 ns;
	
  BEGIN

	gi: for i in 0 to N-1 generate
			 gj: for j in 0 to B-1 generate
				      X_in(i,j) <= X(i)(j);
				  end generate;
		 end generate;
		 
  -- Component Instantiation
   uut: adder_tree port map (clock, resetn, X_in, Yo);
	ck: process
			begin
				wait for OFFSET; -- Wait 100 ns for global reset to finish
				clock_loop: loop -- loop that defines the PLB_clock
					clock <= '0';
					wait for (PERIOD - (PERIOD*DUTY_CYCLE));
					clock <= '1';
					wait for (PERIOD*DUTY_CYCLE);
				end loop clock_loop;
		 end process;
	 
  --  Test Bench Statements
     tb : PROCESS
     BEGIN
		  resetn <= '0';
		  lp: for i in 0 to N-1 loop
					X(i) <= (others => '0');
				end loop;
	     
        wait for OFFSET; -- wait until global set/reset completes
		  resetn <= '1';
       
		  -- This only works for N=8,B=8:
		  X(0) <= x"FA"; X(1) <= x"00"; X(2) <= x"FE"; X(3) <= x"00";
		  X(4) <= x"45"; X(5) <= x"84"; X(6) <= x"AF"; X(7) <= x"BB";
		  wait for PERIOD;
		  X(0) <= x"BE"; X(1) <= x"CA"; X(2) <= x"FA"; X(3) <= x"AB";
		  X(4) <= x"CA"; X(5) <= x"CA"; X(6) <= x"CE"; X(7) <= x"EF";
		  wait for PERIOD;
		  X(0) <= x"BE"; X(1) <= x"01"; X(2) <= x"EF"; X(3) <= x"DE";
		  X(4) <= x"BE"; X(5) <= x"8A"; X(6) <= x"AB"; X(7) <= x"FE";
		  wait for PERIOD;
		  
		  lt: for i in 0 to N-1 loop
					X(i) <= (others => '0');
				end loop;
        wait; -- will wait forever
     END PROCESS tb;
  --  End Test Bench 

  END;
